<?php

namespace IPS\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Musicbox Category Node
 */
class _Category extends \IPS\Node\Model implements \IPS\Node\Permissions
{
	use \IPS\Content\ClubContainer;
	
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;

	/**
	 * @brief	[ActiveRecord] Default Values
	 */
	protected static $defaultValues = NULL;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'musicbox_categories';
	
	/**
	 * @brief	[ActiveRecord] Database Prefix
	 */
	public static $databasePrefix = 'c';
	
	/**
	 * @brief	[Node] Order Database Column
	 */
	public static $databaseColumnOrder = 'position';
	
	/**
	 * @brief	[Node] Parent ID Database Column
	 */
	//public static $databaseColumnParent = 'parent';
	
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'musicbox_cat_manage';
				
	/**
	 * @brief	[Node] Show forms modally?
	 */
	public static $modalForms = false;
	
	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @encode
	 */
	protected static $restrictions = array(
		'app'		=> 'musicbox',
		'module'	=> 'musicbox',
		'prefix' 	=> 'categories_',
		'all'		=> 'categories_manage',
	);
	
	/** 
	 * @brief	[Node] App for permission index
	 */
	public static $permApp = 'musicbox';
	
	/** 
	 * @brief	[Node] Type for permission index
	 */
	public static $permType = 'category';
	
	/**
	 * @brief	The map of permission columns
	 */
	public static $permissionMap = array(
		'view' 		=> 'view',
		'read'		=> 2,
		'add'		=> 3,
		'reply'		=> 4,
		'plist'		=> 5,
		'download'	=> 6,
	);

	/** 
	 * @brief	[Node] Moderator Permission
	 */
	public static $modPerm = 'musicbox_categories';
	
	/**
	 * Columns needed to query for search result / stream view
	 *
	 * @return	array
	 */
	public static function basicDataColumns()
	{
		$return = parent::basicDataColumns();
		$return[] = 'cclub_id';
		return $return;
	}
	
	/**
	 * @brief	Bitwise values for members_bitoptions field
	 */
	public static $bitOptions = array(
		'bitoptions' => array(
			'bitoptions' => array(
				'moderation'			=> 1,	// Require videos to be approved?
				'comments'				=> 2,	// Enable comments?
				'comment_moderation'	=> 4,	// Require comments to be approved?
				'rating'				=> 8, 	// Allow rating
				'sharing'				=> 16,  // Allow sharing
				# 32 is deprecated
				'containerMode'			=> 64,  // Enable container mode
				'lyric'					=> 128,  // Enable lyric
			)
		)
	);	

	/**
	 * @brief	[Node] Title prefix.  If specified, will look for a language key with "{$key}_title" as the key
	 */
	public static $titleLangPrefix = 'musicbox_category_';
	
	
	/**
	 * @brief	[Node] Prefix string that is automatically prepended to permission matrix language strings
	 */
	public static $permissionLangPrefix = 'musicbox_perm_';
		
	/**
	 * @brief	Content Item Class
	 */
	public static $contentItemClass = 'IPS\musicbox\Song';
			
	/**
	 * Load and check permissions
	 *
	 * @param	mixed	$id		ID
	 * @param	string	$perm	Permission Key
	 * @return	static
	 * @throws	\OutOfRangeException
	 */
	public static function loadAndCheckPerms( $id, $perm='view' )
	{
		$node = parent::loadAndCheckPerms( $id, $perm );
		
		if ( !$node->open and !\IPS\Member::loggedIn()->isAdmin() )
		{
			throw new \OutOfRangeException;
		}
		
		return $node;
	}
	
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->open;
	}
	
	/**
	 * Get sort order
	 *
	 * @return	string
	 */
	public function get__sortBy()
	{
		return $this->sortorder;
	}
	
	/**
	 * Get sort order
	 *
	 * @return	string
	 */
	public function get__sortOrder()
	{
		return $this->sortorder == 'song_title' ? 'ASC' : parent::get__sortOrder();
	}
	
	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		$this->open	= $enabled;
	}

	/**
	 * @brief	Cached URL
	 */
	protected $_url	= NULL;
	
	/**
	 * @brief	URL Base
	 */
	public static $urlBase = 'app=musicbox&module=musicbox&controller=songs&id=';
	
	/**
	 * @brief	URL Base
	 */
	public static $urlTemplate = 'musicbox_songs';
	
	/**
	 * @brief	SEO Title Column
	 */
	public static $seoTitleColumn = 'name_furl';

	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$customFields = array();
		foreach ( \IPS\Db::i()->select( 'cf_id', 'musicbox_cfields', NULL, 'cf_position' ) as $fieldId )
		{
			$customFields[ $fieldId ] = \IPS\Member::loggedIn()->language()->addToStack( "musicbox_field_{$fieldId}" );
		}
		
		$form->addTab( 'ctab_settings' );
		$form->addHeader( 'ctab_settings' );
		$form->add( new \IPS\Helpers\Form\Translatable( 'cname', NULL, TRUE, array( 'app' => 'musicbox', 'key' => ( $this->id ? "musicbox_category_{$this->id}" : NULL ) ) ) );
		//$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_containerMode', $this->id ? $this->bitoptions['containerMode'] : FALSE, FALSE, array() ) );		
		// $form->add( new \IPS\Helpers\Form\Node( 'cparent', $this->parent ?: NULL, FALSE, array(
			// 'class'		      => '\IPS\musicbox\Category',
			// 'disabled'	      => false,
			// 'zeroVal'         => 'node_no_parent',
			// 'permissionCheck' => function( $node )
			// {
				// if ( ! isset( \IPS\Request::i()->id ) )
				// {
					// return true;
				// }
				
				// return $node->id != \IPS\Request::i()->id;
			// }
		// ) ) );
		
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_rating', $this->id ? $this->bitoptions['rating'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_sharing', $this->id ? $this->bitoptions['sharing'] : TRUE, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Select( 'csortorder', $this->sortorder ?: 'updated', FALSE, array( 'options' => array( 'updated' => 'sort_updated', 'title' => 'sort_title', 'rating' => 'sort_rating', 'date' => 'sort_date', 'views' => 'sort_num_views' ) ), NULL, NULL, NULL, 'csortorder' ) );
		
		$form->addHeader( 'ctab_cfields' );
		$form->add( new \IPS\Helpers\Form\Radio( 'cfieldpos', $this->fieldpos ?: 'sidebar', FALSE, array( 'options' => array( 'sidebar' => 'mb_fieldpos_sidebar', 'bottom' => 'mb_fieldpos_bottom' ) ), NULL, NULL, NULL, 'cfieldpos' ) );

		if ( !empty( $customFields ) )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'ccfields', $this->id ? explode( ',', $this->_data['cfields'] ) : array(), FALSE, array( 'options' => $customFields, 'multiple' => TRUE ), NULL, NULL, NULL, 'cfields' ) );
		}
		
		if ( \IPS\Settings::i()->tags_enabled )
		{
			$form->addHeader( 'category_tags' );
			$form->add( new \IPS\Helpers\Form\YesNo( 'ctags_disabled', !$this->tags_disabled, FALSE, array( 'togglesOn' => array( 'ctags_noprefixes', 'ctags_predefined' ) ), NULL, NULL, NULL, 'ctags_disabled' ) );
			$form->add( new \IPS\Helpers\Form\YesNo( 'ctags_noprefixes', !$this->tags_noprefixes, FALSE, array(), NULL, NULL, NULL, 'ctags_noprefixes' ) );
			
			if ( !\IPS\Settings::i()->tags_open_system )
			{
				$form->add( new \IPS\Helpers\Form\Text( 'ctags_predefined', $this->tags_predefined, FALSE, array( 'autocomplete' => array( 'unique' => 'true' ), 'nullLang' => 'ctags_predefined_unlimited' ), NULL, NULL, NULL, 'ctags_predefined' ) );
			}
		}
		
		$form->addHeader( 'mbconf_header_moderation' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_moderation', $this->bitoptions['moderation'], FALSE, array() ) );	
				
		$form->addHeader( 'mbconf_header_comment' );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_comments', $this->id ? $this->bitoptions['comments'] : TRUE, FALSE, array( 'togglesOn' => array( 'mb_bitoptions_comment_moderation' ) ), NULL, NULL, NULL, 'mb_bitoptions_comments' ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_comment_moderation', $this->bitoptions['comment_moderation'], FALSE, array(), NULL, NULL, NULL, 'mb_bitoptions_comment_moderation' ) );		
		
		$form->addHeader( 'mbconf_header_lyric' );			
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_lyric', $this->bitoptions['lyric'], FALSE, array(), NULL, NULL, NULL, 'mb_bitoptions_lyric' ) );		
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function formatFormValues( $values )
	{
		if ( !$this->id )
		{
			$this->save();
		}
		
		/* Save lang for title */
		foreach ( array( 'cname' => "musicbox_category_{$this->id}" ) as $fieldKey => $langKey )
		{
			if ( isset( $values[ $fieldKey ] ) )
			{
				\IPS\Lang::saveCustom( 'musicbox', $langKey, $values[ $fieldKey ] );
				
				if ( $fieldKey === 'cname' )
				{
					$this->name = $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ];
					$this->name_furl = \IPS\Http\Url::seoTitle( $values[ $fieldKey ][ \IPS\Lang::defaultLanguage() ] );
				}
				
				unset( $values[ $fieldKey ] );
			}
		}		

		/* Save bitoptions */
		foreach ( array( 'moderation', 'comments', 'comment_moderation', 'rating', 'sharing', 'containerMode', 'lyric' ) as $k )
		{
			if ( isset( $values[ "mb_bitoptions_{$k}" ] ) )
			{
				$this->bitoptions[ $k ] = $values["mb_bitoptions_{$k}"];
				unset( $values["mb_bitoptions_{$k}"] );
			}
		}
		
		/* Inverted for legacy reasons */
		foreach ( array( 'ctags_disabled', 'ctags_noprefixes' ) as $k )
		{
			if ( isset( $values[ $k ] ) )
			{
				$values[ $k ] = !$values[ $k ];
			}
		}		
		
		/* Avoid "parent cannot be null" error if no parent selected. */
		// if ( isset( $values['cparent'] ) )
		// {
			// $values['cparent'] = $values['cparent'] ? \intval( $values['cparent']->id ) : 0;
		// }		
		
		/* return */
		return $values;
	}

	/* !Clubs */
	
	/**
	 * Get acp language string
	 *
	 * @return	string
	 */
	public static function clubAcpTitle()
	{
		return 'musicbox_categories';
	}
	
	/**
	 * Set form for creating a node of this type in a club
	 *
	 * @param	\IPS\Helpers\Form	$form	Form object
	 * @return	void
	 */
	public function clubForm( \IPS\Helpers\Form $form )
	{
		$itemClass = static::$contentItemClass;
		$form->class = "ipsForm_horizontal";

		$form->add( new \IPS\Helpers\Form\Text( 'club_node_name', $this->_id ? $this->_title : \IPS\Member::loggedIn()->language()->addToStack( $itemClass::$title . '_pl' ), TRUE, array( 'maxLength' => 255 ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_comments', $this->id ? $this->bitoptions['comments'] : TRUE, FALSE, array( 'togglesOn' => array( 'mb_bitoptions_comment_moderation' ) ), NULL, NULL, NULL, 'mb_bitoptions_comments' ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'mb_bitoptions_lyric', $this->bitoptions['lyric'], FALSE, array(), NULL, NULL, NULL, 'mb_bitoptions_lyric' ) );		
	}
	
	public function _saveClubForm( \IPS\Member\Club $club, $values )
	{
		$this->bitoptions['rating'] = true;
		$this->bitoptions['sharing'] = true;
		
		foreach ( array( 'comments', 'lyric' ) as $k )
		{
			$this->bitoptions[ $k ] = $values[ 'mb_bitoptions_' . $k ];
		}	
		
		if ( $values['club_node_name'] )
		{
			$this->name_furl = \IPS\Http\Url\Friendly::seoTitle( $values['club_node_name'] );
		}		
	}	
	/**
	 * Get custom fields
	 *
	 * @return	array
	 */
	protected function get_cfields()
	{
		$fields = array();
		foreach ( explode( ',', $this->_data['cfields'] ) as $fieldId )
		{
			try
			{
				$fields[ $fieldId ] = \IPS\musicbox\Field::load( $fieldId );
			}
			catch ( \OutOfRangeException $e ) { }
		}
		
		return $fields;
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		parent::delete();
		
		foreach ( array( 'cname' => "musicbox_category_{$this->id}" ) as $fieldKey => $langKey )
		{
			\IPS\Lang::deleteCustom( 'musicbox', $langKey );
			\IPS\Lang::deleteCustom( 'core', $langKey ); // Remove custom lang for cloned catergory, because it was saved for core.
		}		
	}	
}