<?php

namespace IPS\musicbox;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Feed Import Node
 */
class _Feed extends \IPS\Node\Model
{
	/**
	 * @brief	[ActiveRecord] Multiton Store
	 */
	protected static $multitons;
	
	/**
	 * @brief	[ActiveRecord] Database Table
	 */
	public static $databaseTable = 'musicbox_rss_import';
	
	/**
	 * @brief	Database Prefix
	 */
	public static $databasePrefix = 'mbrss_import_';
				
	/**
	 * @brief	[Node] Node Title
	 */
	public static $nodeTitle = 'mbrss_import';
	
	/**
	 * @brief	[Node] ACP Restrictions
	 * @code
	 	array(
	 		'app'		=> 'core',				// The application key which holds the restrictrions
	 		'module'	=> 'foo',				// The module key which holds the restrictions
	 		'map'		=> array(				// [Optional] The key for each restriction - can alternatively use "prefix"
	 			'add'			=> 'foo_add',
	 			'edit'			=> 'foo_edit',
	 			'permissions'	=> 'foo_perms',
	 			'delete'		=> 'foo_delete'
	 		),
	 		'all'		=> 'foo_manage',		// [Optional] The key to use for any restriction not provided in the map (only needed if not providing all 4)
	 		'prefix'	=> 'foo_',				// [Optional] Rather than specifying each  key in the map, you can specify a prefix, and it will automatically look for restrictions with the key "[prefix]_add/edit/permissions/delete"
	 * @endcode
	 */
	protected static $restrictions = array(
		'app'		=> 'musicbox',
		'module'	=> 'musicbox',
		'prefix'	=> 'rss_'
	);
	
	/**
	 * [Node] Get title
	 *
	 * @return	string
	 */
	protected function get__title()
	{
		return $this->title;
	}
	
	/**
	 * [Node] Get description
	 *
	 * @return	string
	 */
	protected function get__description()
	{
		return $this->url;
	}
	
	/**
	 * [Node] Get whether or not this node is enabled
	 *
	 * @note	Return value NULL indicates the node cannot be enabled/disabled
	 * @return	bool|null
	 */
	protected function get__enabled()
	{
		return $this->enabled;
	}

	/**
	 * [Node] Set whether or not this node is enabled
	 *
	 * @param	bool|int	$enabled	Whether to set it enabled or disabled
	 * @return	void
	 */
	protected function set__enabled( $enabled )
	{
		if ( $enabled )
		{
			\IPS\Db::i()->update( 'core_tasks', array( 'enabled' => 1 ), array( '`key`=?', 'mbrssimport' ) );
		}
		$this->enabled = $enabled;
	}
	
	/**
	 * [Node] Get buttons to display in tree
	 * Example code explains return value
	 *
	 * @code
	 	array(
	 		array(
	 			'icon'	=>	'plus-circle', // Name of FontAwesome icon to use
	 			'title'	=> 'foo',		// Language key to use for button's title parameter
	 			'link'	=> \IPS\Http\Url::internal( 'app=foo...' )	// URI to link to
	 			'class'	=> 'modalLink'	// CSS Class to use on link (Optional)
	 		),
	 		...							// Additional buttons
	 	);
	 * @endcode
	 * @param	string	$url		Base URL
	 * @param	bool	$subnode	Is this a subnode?
	 * @return	array
	 */
	public function getButtons( $url, $subnode=FALSE )
	{
		$buttons = parent::getButtons( $url, $subnode );
		
		if ( \IPS\Member::loggedIn()->hasAcpRestriction('rss_run') )
		{
			$buttons = array_merge( array( 'run' => array(
				'icon'	=> 'refresh',
				'title'	=> 'rss_run',
				'link'	=> $url->setQueryString( array( 'do' => 'run', 'id' => $this->_id ) )->csrf()
			) ), $buttons );
		}
		
		return $buttons;
	}
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form	The form
	 * @return	void
	 */
	public function form( &$form )
	{
		$form->addHeader('mbrss_import_url');
		$form->add( new \IPS\Helpers\Form\Url( 'mbrss_import_url', $this->url, TRUE ) );
		$form->add( new \IPS\Helpers\Form\Text( 'mbrss_import_auth_user', $this->auth_user ) );
		$form->add( new \IPS\Helpers\Form\Password( 'mbrss_import_auth_pass', $this->auth_pass ) );
		$form->addHeader('mbrss_import_details');
		$form->add( new \IPS\Helpers\Form\Node( 'mbrss_import_genre_id', $this->genre_id, TRUE, array( 'class' => 'IPS\musicbox\Category' ) ) );
		$form->add( new \IPS\Helpers\Form\Member( 'mbrss_import_mid', \IPS\Member::load( $this->mid ), TRUE ) );
		$form->add( new \IPS\Helpers\Form\Text( 'mbrss_import_artist', $this->artist, FALSE ) );
	}
	
	/**
	 * [Node] Format form values from add/edit form for save
	 *
	 * @param	array	$values	Values from the form
	 * @return	array
	 */
	public function formatFormValues( $values )
	{
		if( isset( $values['mbrss_import_url'] ) )
		{
			try
			{
				$request = $values['mbrss_import_url']->request();
				
				if ( $values['mbrss_import_auth_user'] or $values['mbrss_import_auth_pass'] )
				{
					$request = $request->login( $values['mbrss_import_auth_user'], $values['mbrss_import_auth_pass'] );
				}
				
				$response = $request->get();
				
				if ( $response->httpResponseCode == 401 )
				{
					throw new \DomainException('mbrss_import_auth');
				}
				
				$response = $response->decodeXml();
				if ( !( $response instanceof \IPS\Xml\Rss ) and !( $response instanceof \IPS\Xml\Atom ) )
				{
					throw new \DomainException('mbrss_import_invalid');
				}
			}
			catch ( \IPS\Http\Request\Exception $e )
			{
				throw new \DomainException( 'form_url_bad' );
			}
			catch ( \ErrorException $e )
			{
				throw new \DomainException( 'mbrss_import_invalid' );
			}

			$values['title'] = (string) $request->get()->decodeXml()->channel->title;
		}
		
		if( isset( $values['mbrss_import_genre_id'] ) )
		{
			$values['genre_id'] = $values['mbrss_import_genre_id']->_id;
			unset( $values['mbrss_import_genre_id'] );
		}

		if( isset( $values['mbrss_import_mid'] ) )
		{
			$values['mid'] = $values['mbrss_import_mid']->member_id;
			unset( $values['mbrss_import_mid'] );
		}
		
		if( isset( $values['mbrss_import_artist'] ) )
		{
			$values['artist'] = $values['mbrss_import_artist'];
			unset( $values['mbrss_import_artist'] );
		}		

		if( isset( $values['mbrss_import_auth_user'] ) OR isset( $values['mbrss_import_auth_pass'] ) )
		{
			$values['auth'] = ( $values['mbrss_import_auth_user'] or $values['mbrss_import_auth_pass'] );
		}
		
		return $values;
	}

	/**
	 * [Node] Perform actions after saving the form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function postSaveForm( $values )
	{
		$this->run();
	}

	/**
	 * Run Import
	 *
	 * @return	void
	 * @throws	\IPS\Http\Url\Exception
	 */
	public function run()
	{
		/* Skip this if the member is restricted from posting */
		if( \IPS\Member::load( $this->mid )->restrict_post or \IPS\Member::load( $this->mid )->members_bitoptions['unacknowledged_warnings'] )
		{
			return;
		}

		$previouslyImportedGuids = iterator_to_array( \IPS\Db::i()->select( 'mbrss_imported_guid', 'musicbox_rss_imported', array( 'mbrss_imported_impid=?', $this->id ) ) );
		
		$request = \IPS\Http\Url::external( $this->url )->request();
		if ( $this->auth )
		{
			$request = $request->login( $this->auth_user, $this->auth_pass );
		}
		$request = $request->get();
		
		$container = \IPS\musicbox\Category::load( $this->genre_id );
		
		$i = 0;
		$inserts = array();
		$request = $request->decodeXml();

		if( !( $request instanceof \IPS\Xml\RssOne ) AND !( $request instanceof \IPS\Xml\Rss ) AND !( $request instanceof \IPS\Xml\Atom ) )
		{
			throw new \RuntimeException( 'mbrss_import_invalid' );
		}


		$allItems = $request->musicboxGetItems( $this->id );
		$allItems = array_reverse($allItems);

		foreach ( $allItems as $guid => $article )
		{
			if ( !\in_array( $guid, $previouslyImportedGuids ) )
			{
				$song = \IPS\musicbox\Song::createItem( \IPS\Member::load( $this->mid ), NULL, $article['date'], $container );
				$song->title = $article['title'];
				$song->title_furl = \IPS\Http\Url::seoTitle( $article['title'] );
				$feedcontent = preg_replace('/<p\b[^>]*>(.*?)<\/p>/i', '$1', nl2br(htmlspecialchars_decode($article['content'])));
				$song->content = $feedcontent;
				if ( $article['author'] )
				{
					$artist = preg_replace("/[^@\s]*@[^@\s]*\.[^@\s]*/", "", (string) $article['author']);
					$artist = strip_tags($artist);
					$artist = str_replace("(", "", $artist);
					$artist = str_replace(")", "", $artist);
					$artist = trim($artist);
				}
				$song->singer = $this->artist ? $this->artist : $artist;
				$song->external = 1;
				if ( $article['media'] )
				{
					$song->file = $article['media'];
					$song->filesize = \IPS\Application::load('musicbox')->remotefileSize($article['media']);
				}
				$song->save();

				\IPS\Db::i()->insert( 'musicbox_rss_imported', array(
					'mbrss_imported_guid'	=> $guid,
					'mbrss_imported_id'		=> $song->id,
					'mbrss_imported_impid'	=> $this->id,
				) );
			
							
				$i++;
				
				if ( $i >= 10 )
				{
					break;
				}	
			}
		}
		
		$this->last_import = time();
		$this->save();

		$container->setLastComment();
		$container->save();
	}
	
	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		\IPS\Db::i()->delete( 'musicbox_rss_imported', array( "mbrss_imported_impid=?", $this->id ) );
		
		return parent::delete();
	}

	/**
	 * Search
	 *
	 * @param	string		$column	Column to search
	 * @param	string		$query	Search query
	 * @param	string|null	$order	Column to order by
	 * @param	mixed		$where	Where clause
	 * @return	array
	 */
	public static function search( $column, $query, $order=NULL, $where=array() )
	{	
		if ( $column === '_title' )
		{
			$column = 'mbrss_import_title';
		}
		if ( $order === '_title' )
		{
			$order = 'mbrss_import_title';
		}
		return parent::search( $column, $query, $order, $where );
	}
}